@@ -13,12 +13,15 @@ extern "C" {
 #include <cont.h> // g_pcont declaration
 
 extern bool timeshift64_is_set;
+extern uint32_t sntp_real_timestamp;
 
 bool can_yield();
 void esp_yield();
 void esp_schedule();
 void tune_timeshift64 (uint64_t now_us);
 void disable_extra4k_at_link_time (void) __attribute__((noinline));
+bool sntp_set_timezone_in_seconds(int32_t timezone);
+void __real_system_restart_local() __attribute__((noreturn));
 
 uint32_t sqrt32 (uint32_t n);
 uint32_t crc32 (const void* data, size_t length, uint32_t crc = 0xffffffff);
