@@ -28,6 +28,10 @@
  *                        wrappers that use critical section protection macros
  *                        and static core functions that assume they are
  *                        running in a protected con text. Thanks @devyte
+ * R.Hempel 2020-01-07 - Add support for Fragmentation metric - See Issue 14
+ * R.Hempel 2020-01-12 - Use explicitly sized values from stdint.h - See Issue 15
+ * R.Hempel 2020-01-20 - Move metric functions back to umm_info - See Issue 29
+ * R.Hempel 2020-02-01 - Macro functions are uppercased - See Issue 34
  * ----------------------------------------------------------------------------
  */
 
@@ -41,17 +45,17 @@
 /*
  * Added for using with Arduino ESP8266 and handling renameing to umm_malloc.cpp
  */
-
 #define BUILD_UMM_MALLOC_C
 
 extern "C" {
 
 #include <stdio.h>
+#include <stdint.h>
+#include <stddef.h>
 #include <string.h>
 
-#include "umm_malloc.h"
-
 #include "umm_malloc_cfg.h"   /* user-dependent */
+#include "umm_malloc.h"
 
 /* Use the default DBGLOG_LEVEL and DBGLOG_FUNCTION */
 
@@ -61,13 +65,25 @@ extern "C" {
 
 #include "dbglog/dbglog.h"
 
+//C This change is new in upstream umm_malloc.I think this would have created a
+//C breaking change. Keeping the old #define method in umm_malloc_cfg.h.
+//C I don't see a simple way of making it work. We would have to run code before
+//C the SDK has run to set a value for uint32_t UMM_MALLOC_CFG_HEAP_SIZE.
+//C On the other hand, a manual call to umm_init() before anything else has had a
+//C chance to run would mean that all those calls testing to see if the heap has
+//C been initialized at every umm_malloc API could be removed.
+//C
+//C before starting the NON OS SDK
+//C extern void *UMM_MALLOC_CFG_HEAP_ADDR;
+//C extern uint32_t UMM_MALLOC_CFG_HEAP_SIZE;
+
 #include "umm_local.h"      // target-dependent supplemental
 
 /* ------------------------------------------------------------------------- */
 
 UMM_H_ATTPACKPRE typedef struct umm_ptr_t {
-  unsigned short int next;
-  unsigned short int prev;
+  uint16_t next;
+  uint16_t prev;
 } UMM_H_ATTPACKSUF umm_ptr;
 
 
@@ -77,29 +93,36 @@ UMM_H_ATTPACKPRE typedef struct umm_block_t {
   } header;
   union {
     umm_ptr free;
-    unsigned char data[4];
+    uint8_t data[4];
   } body;
 } UMM_H_ATTPACKSUF umm_block;
 
-#define UMM_FREELIST_MASK (0x8000)
-#define UMM_BLOCKNO_MASK  (0x7FFF)
+#define UMM_FREELIST_MASK ((uint16_t)(0x8000))
+#define UMM_BLOCKNO_MASK  ((uint16_t)(0x7FFF))
 
 /* ------------------------------------------------------------------------- */
 
 umm_block *umm_heap = NULL;
-unsigned short int umm_numblocks = 0;
+uint16_t umm_numblocks = 0;
 
-#define UMM_NUMBLOCKS (umm_numblocks)
+#define UMM_NUMBLOCKS  (umm_numblocks)
+#define UMM_BLOCK_LAST (UMM_NUMBLOCKS - 1)
 
-/* ------------------------------------------------------------------------ */
+/* -------------------------------------------------------------------------
+ * These macros evaluate to the address of the block and data respectively
+ */
 
 #define UMM_BLOCK(b)  (umm_heap[b])
+#define UMM_DATA(b)   (UMM_BLOCK(b).body.data)
+
+/* -------------------------------------------------------------------------
+ * These macros evaluate to the index of the block - NOT the address!!!
+ */
 
 #define UMM_NBLOCK(b) (UMM_BLOCK(b).header.used.next)
 #define UMM_PBLOCK(b) (UMM_BLOCK(b).header.used.prev)
 #define UMM_NFREE(b)  (UMM_BLOCK(b).body.free.next)
 #define UMM_PFREE(b)  (UMM_BLOCK(b).body.free.prev)
-#define UMM_DATA(b)   (UMM_BLOCK(b).body.data)
 
 /* -------------------------------------------------------------------------
  * There are additional files that may be included here - normally it's
@@ -116,7 +139,7 @@ unsigned short int umm_numblocks = 0;
 
 /* ------------------------------------------------------------------------ */
 
-static unsigned short int umm_blocks( size_t size ) {
+static uint16_t umm_blocks( size_t size ) {
 
   /*
    * The calculation of the block size is not too difficult, but there are
@@ -149,9 +172,9 @@ static unsigned short int umm_blocks( size_t size ) {
  *
  * Note that free pointers are NOT modified by this function.
  */
-static void umm_split_block( unsigned short int c,
-    unsigned short int blocks,
-    unsigned short int new_freemask ) {
+static void umm_split_block( uint16_t c,
+    uint16_t blocks,
+    uint16_t new_freemask ) {
 
   UMM_NBLOCK(c+blocks) = (UMM_NBLOCK(c) & UMM_BLOCKNO_MASK) | new_freemask;
   UMM_PBLOCK(c+blocks) = c;
@@ -162,7 +185,7 @@ static void umm_split_block( unsigned short int c,
 
 /* ------------------------------------------------------------------------ */
 
-static void umm_disconnect_from_free_list( unsigned short int c ) {
+static void umm_disconnect_from_free_list( uint16_t c ) {
   /* Disconnect this block from the FREE list */
 
   UMM_NFREE(UMM_PFREE(c)) = UMM_NFREE(c);
@@ -174,13 +197,17 @@ static void umm_disconnect_from_free_list( unsigned short int c ) {
 }
 
 /* ------------------------------------------------------------------------
- * The umm_assimilate_up() function assumes that UMM_NBLOCK(c) does NOT
- * have the UMM_FREELIST_MASK bit set!
+ * The umm_assimilate_up() function does not assume that UMM_NBLOCK(c)
+ * has the UMM_FREELIST_MASK bit set. It only assimilates up if the
+ * next block is free.
  */
 
-static void umm_assimilate_up( unsigned short int c ) {
+static void umm_assimilate_up( uint16_t c ) {
 
   if( UMM_NBLOCK(UMM_NBLOCK(c)) & UMM_FREELIST_MASK ) {
+
+    UMM_FRAGMENTATION_METRIC_REMOVE( UMM_NBLOCK(c) );
+
     /*
      * The next block is a free block, so assimilate up and remove it from
      * the free list
@@ -201,14 +228,30 @@ static void umm_assimilate_up( unsigned short int c ) {
 
 /* ------------------------------------------------------------------------
  * The umm_assimilate_down() function assumes that UMM_NBLOCK(c) does NOT
- * have the UMM_FREELIST_MASK bit set!
+ * have the UMM_FREELIST_MASK bit set. In other words, try to assimilate
+ * up before assimilating down.
  */
 
-static unsigned short int umm_assimilate_down( unsigned short int c, unsigned short int freemask ) {
+static uint16_t umm_assimilate_down( uint16_t c, uint16_t freemask ) {
+
+  // We are going to assimilate down to the previous block because
+  // it was free, so remove it from the fragmentation metric
+
+  UMM_FRAGMENTATION_METRIC_REMOVE(UMM_PBLOCK(c));
 
   UMM_NBLOCK(UMM_PBLOCK(c)) = UMM_NBLOCK(c) | freemask;
   UMM_PBLOCK(UMM_NBLOCK(c)) = UMM_PBLOCK(c);
 
+  if (freemask) {
+      // We are going to free the entire assimilated block
+      // so add it to the fragmentation metric. A good
+      // compiler will optimize away the empty if statement
+      // when UMM_INFO is not defined, so don't worry about
+      // guarding it.
+
+      UMM_FRAGMENTATION_METRIC_ADD(UMM_PBLOCK(c));
+  }
+
   return( UMM_PBLOCK(c) );
 }
 
@@ -221,62 +264,54 @@ void umm_init( void ) {
   memset(umm_heap, 0x00, UMM_MALLOC_CFG_HEAP_SIZE);
 
   /* setup initial blank heap structure */
-  {
-    /* index of the 0th `umm_block` */
-    const unsigned short int block_0th = 0;
-    /* index of the 1st `umm_block` */
-    const unsigned short int block_1th = 1;
-    /* index of the latest `umm_block` */
-    const unsigned short int block_last = UMM_NUMBLOCKS - 1;
+    UMM_FRAGMENTATION_METRIC_INIT();
 
     /* init ummStats.free_blocks */
 #if defined(UMM_STATS) || defined(UMM_STATS_FULL)
 #if defined(UMM_STATS_FULL)
     ummStats.free_blocks_min =
-    ummStats.free_blocks_isr_min  =
+    ummStats.free_blocks_isr_min  = UMM_NUMBLOCKS - 2;
+#endif
+#ifndef UMM_INLINE_METRICS
+    ummStats.free_blocks = UMM_NUMBLOCKS - 2;
 #endif
-    ummStats.free_blocks = block_last;
 #endif
 
-    /* setup the 0th `umm_block`, which just points to the 1st */
-    UMM_NBLOCK(block_0th) = block_1th;
-    UMM_NFREE(block_0th)  = block_1th;
-    UMM_PFREE(block_0th)  = block_1th;
+    /* Set up umm_block[0], which just points to umm_block[1] */
+    UMM_NBLOCK(0) = 1;
+    UMM_NFREE(0)  = 1;
+    UMM_PFREE(0)  = 1;
 
     /*
      * Now, we need to set the whole heap space as a huge free block. We should
-     * not touch the 0th `umm_block`, since it's special: the 0th `umm_block`
-     * is the head of the free block list. It's a part of the heap invariant.
+     * not touch umm_block[0], since it's special: umm_block[0] is the head of
+     * the free block list. It's a part of the heap invariant.
      *
      * See the detailed explanation at the beginning of the file.
-     */
-
-    /*
-     * 1th `umm_block` has pointers:
      *
-     * - next `umm_block`: the latest one
-     * - prev `umm_block`: the 0th
+     * umm_block[1] has pointers:
+     *
+     * - next `umm_block`: the last one umm_block[n]
+     * - prev `umm_block`: umm_block[0]
      *
      * Plus, it's a free `umm_block`, so we need to apply `UMM_FREELIST_MASK`
      *
-     * And it's the last free block, so the next free block is 0.
+     * And it's the last free block, so the next free block is 0 which marks
+     * the end of the list. The previous block and free block pointer are 0
+     * too, there is no need to initialize these values due to the init code
+     * that memsets the entire umm_ space to 0.
      */
-    UMM_NBLOCK(block_1th) = block_last | UMM_FREELIST_MASK;
-    UMM_NFREE(block_1th)  = 0;
-    UMM_PBLOCK(block_1th) = block_0th;
-    UMM_PFREE(block_1th)  = block_0th;
+    UMM_NBLOCK(1) = UMM_BLOCK_LAST | UMM_FREELIST_MASK;
 
     /*
-     * latest `umm_block` has pointers:
+     * Last umm_block[n] has the next block index at 0, meaning it's
+     * the end of the list, and the previous block is umm_block[1].
      *
-     * - next `umm_block`: 0 (meaning, there are no more `umm_blocks`)
-     * - prev `umm_block`: the 1st
-     *
-     * It's not a free block, so we don't touch NFREE / PFREE at all.
+     * The last block is a special block and can never be part of the
+     * free list, so its pointers are left at 0 too.
      */
-    UMM_NBLOCK(block_last) = 0;
-    UMM_PBLOCK(block_last) = block_1th;
-  }
+
+    UMM_PBLOCK(UMM_BLOCK_LAST) = 1;
 }
 
 /* ------------------------------------------------------------------------
@@ -286,7 +321,7 @@ void umm_init( void ) {
 
 static void umm_free_core( void *ptr ) {
 
-  unsigned short int c;
+  uint16_t c;
 
   STATS__FREE_REQUEST(id_free);
   /*
@@ -300,7 +335,7 @@ static void umm_free_core( void *ptr ) {
 
   /* Figure out which block we're in. Note the use of truncated division... */
 
-  c = (((char *)ptr)-(char *)(&(umm_heap[0])))/sizeof(umm_block);
+  c = (((uintptr_t)ptr)-(uintptr_t)(&(umm_heap[0])))/sizeof(umm_block);
 
   DBGLOG_DEBUG( "Freeing block %6d\n", c );
 
@@ -315,7 +350,7 @@ static void umm_free_core( void *ptr ) {
 
   if( UMM_NBLOCK(UMM_PBLOCK(c)) & UMM_FREELIST_MASK ) {
 
-    DBGLOG_DEBUG( "Assimilate down to next block, which is FREE\n" );
+    DBGLOG_DEBUG( "Assimilate down to previous block, which is FREE\n" );
 
     c = umm_assimilate_down(c, UMM_FREELIST_MASK);
   } else {
@@ -323,6 +358,7 @@ static void umm_free_core( void *ptr ) {
      * The previous block is not a free block, so add this one to the head
      * of the free list
      */
+    UMM_FRAGMENTATION_METRIC_ADD(c);
 
     DBGLOG_DEBUG( "Just add to head of free list\n" );
 
@@ -368,13 +404,13 @@ void umm_free( void *ptr ) {
  */
 
 static void *umm_malloc_core( size_t size ) {
-  unsigned short int blocks;
-  unsigned short int blockSize = 0;
+  uint16_t blocks;
+  uint16_t blockSize = 0;
 
-  unsigned short int bestSize;
-  unsigned short int bestBlock;
+  uint16_t bestSize;
+  uint16_t bestBlock;
 
-  unsigned short int cf;
+  uint16_t cf;
 
   STATS__ALLOC_REQUEST(id_malloc, size);
 
@@ -422,6 +458,9 @@ static void *umm_malloc_core( size_t size ) {
   POISON_CHECK_NEIGHBORS(cf);
 
   if( UMM_NBLOCK(cf) & UMM_BLOCKNO_MASK && blockSize >= blocks ) {
+
+    UMM_FRAGMENTATION_METRIC_REMOVE(cf);
+
     /*
      * This is an existing block in the memory heap, we just need to split off
      * what we need, unlink it from the free list and mark it as in use, and
@@ -436,8 +475,8 @@ static void *umm_malloc_core( size_t size ) {
       /* Disconnect this block from the FREE list */
 
       umm_disconnect_from_free_list( cf );
-
     } else {
+
       /* It's not an exact fit and we need to split off a block. */
       DBGLOG_DEBUG( "Allocating %6d blocks starting at %6d - existing\n", blocks, cf );
 
@@ -447,6 +486,8 @@ static void *umm_malloc_core( size_t size ) {
        */
       umm_split_block( cf, blocks, UMM_FREELIST_MASK /*new block is free*/ );
 
+      UMM_FRAGMENTATION_METRIC_ADD(UMM_NBLOCK(cf));
+
       /*
        * `umm_split_block()` does not update the free pointers (it affects
        * only free flags), but effectively we've just moved beginning of the
@@ -518,12 +559,12 @@ void *umm_malloc( size_t size ) {
 void *umm_realloc( void *ptr, size_t size ) {
   UMM_CRITICAL_DECL(id_realloc);
 
-  unsigned short int blocks;
-  unsigned short int blockSize;
-  unsigned short int prevBlockSize = 0;
-  unsigned short int nextBlockSize = 0;
+  uint16_t blocks;
+  uint16_t blockSize;
+  uint16_t prevBlockSize = 0;
+  uint16_t nextBlockSize = 0;
 
-  unsigned short int c;
+  uint16_t c;
 
   size_t curSize;
 
@@ -551,7 +592,6 @@ void *umm_realloc( void *ptr, size_t size ) {
    * we should operate the same as free.
    */
 
-
   if( 0 == size ) {
     DBGLOG_DEBUG( "realloc to 0 size, just free the block\n" );
     STATS__ZERO_ALLOC_REQUEST(id_realloc, size);
@@ -576,7 +616,7 @@ void *umm_realloc( void *ptr, size_t size ) {
 
   /* Figure out which block we're in. Note the use of truncated division... */
 
-  c = (((char *)ptr)-(char *)(&(umm_heap[0])))/sizeof(umm_block);
+  c = (((uintptr_t)ptr)-(uintptr_t)(&(umm_heap[0])))/sizeof(umm_block);
 
   /* Figure out how big this block is ... the free bit is not set :-) */
 
@@ -607,6 +647,21 @@ void *umm_realloc( void *ptr, size_t size ) {
 
   DBGLOG_DEBUG( "realloc blocks %d blockSize %d nextBlockSize %d prevBlockSize %d\n", blocks, blockSize, nextBlockSize, prevBlockSize );
 
+//C With each upstream update this section should be reevaluated.
+/*C
+ *
+ * The `#if defined(UMM_REALLOC_MINIMIZE_COPY)` section tracks the content of
+ * the upstream with some local macros added. Back when I made my 1st update to
+ * umm_malloc PR, I found the upstream had been refactored and removed the
+ * defragmenting properties that were originally present. It took some looking
+ * to see the logic, it didn't have any comments to make it stand out.
+ *
+ * I added the `#elif defined(UMM_REALLOC_DEFRAG)` to recreate and preserve the
+ * defragmenting functionality that was lost. This is the default build option
+ * we have set in `umm_malloc_cfg.h`. I have not done any structured testing to
+ * confirm; however, I think this to be the best option when considering the
+ * amount of reallocates that can occur with the Strings library.
+ */
 #if defined(UMM_REALLOC_MINIMIZE_COPY)
   /*
    * Ok, now that we're here we know how many blocks we want and the current
@@ -616,29 +671,53 @@ void *umm_realloc( void *ptr, size_t size ) {
    * 1. If the new block is the same size or smaller than the current block do
    *    nothing.
    * 2. If the next block is free and adding it to the current block gives us
-   *    enough memory, assimilate the next block.
-   * 3. If the prev block is free and adding it to the current block gives us
+   *    EXACTLY enough memory, assimilate the next block. This avoids unwanted
+   *    fragmentation of free memory.
+   *
+   * The following cases may be better handled with memory copies to reduce
+   * fragmentation
+   *
+   * 3. If the previous block is NOT free and the next block is free and
+   *    adding it to the current block gives us enough memory, assimilate
+   *    the next block. This may introduce a bit of fragmentation.
+   * 4. If the prev block is free and adding it to the current block gives us
    *    enough memory, remove the previous block from the free list, assimilate
    *    it, copy to the new block.
-   * 4. If the prev and next blocks are free and adding them to the current
+   * 5. If the prev and next blocks are free and adding them to the current
    *    block gives us enough memory, assimilate the next block, remove the
    *    previous block from the free list, assimilate it, copy to the new block.
-   * 5. Otherwise try to allocate an entirely new block of memory. If the
+   * 6. Otherwise try to allocate an entirely new block of memory. If the
    *    allocation works free the old block and return the new pointer. If
    *    the allocation fails, return NULL and leave the old block intact.
    *
+   * TODO: Add some conditional code to optimise for less fragmentation
+   *       by simply allocating new memory if we need to copy anyways.
+   *
    * All that's left to do is decide if the fit was exact or not. If the fit
    * was not exact, then split the memory block so that we use only the requested
    * number of blocks and add what's left to the free list.
    */
+
+    //  Case 1 - block is same size or smaller
     if (blockSize >= blocks) {
         DBGLOG_DEBUG( "realloc the same or smaller size block - %i, do nothing\n", blocks );
         /* This space intentionally left blank */
-    } else if ((blockSize + nextBlockSize) >= blocks) {
+
+    //  Case 2 - block + next block fits EXACTLY
+    } else if ((blockSize + nextBlockSize) == blocks) {
+        DBGLOG_DEBUG( "exact realloc using next block - %i\n", blocks );
+        umm_assimilate_up( c );
+        STATS__FREE_BLOCKS_UPDATE( - nextBlockSize );
+        blockSize += nextBlockSize;
+
+    //  Case 3 - prev block NOT free and block + next block fits
+    } else if ((0 == prevBlockSize) && (blockSize + nextBlockSize) >= blocks) {
         DBGLOG_DEBUG( "realloc using next block - %i\n", blocks );
         umm_assimilate_up( c );
         STATS__FREE_BLOCKS_UPDATE( - nextBlockSize );
         blockSize += nextBlockSize;
+
+    //  Case 4 - prev block + block fits
     } else if ((prevBlockSize + blockSize) >= blocks) {
         DBGLOG_DEBUG( "realloc using prev block - %i\n", blocks );
         umm_disconnect_from_free_list( UMM_PBLOCK(c) );
@@ -650,6 +729,7 @@ void *umm_realloc( void *ptr, size_t size ) {
         memmove( (void *)&UMM_DATA(c), ptr, curSize );
         ptr = (void *)&UMM_DATA(c);
         UMM_CRITICAL_RESUME(id_realloc);
+    //  Case 5 - prev block + block + next block fits
     } else if ((prevBlockSize + blockSize + nextBlockSize) >= blocks) {
         DBGLOG_DEBUG( "realloc using prev and next block - %d\n", blocks );
         umm_assimilate_up( c );
@@ -670,6 +750,8 @@ void *umm_realloc( void *ptr, size_t size ) {
         memmove( (void *)&UMM_DATA(c), ptr, curSize );
         ptr = (void *)&UMM_DATA(c);
         UMM_CRITICAL_RESUME(id_realloc);
+
+    //  Case 6 - default is we need to realloc a new block
     } else {
         DBGLOG_DEBUG( "realloc a completely new block %i\n", blocks );
         void *oldptr = ptr;
