@@ -8,25 +8,27 @@
 #ifndef UMM_MALLOC_H
 #define UMM_MALLOC_H
 
-/* ------------------------------------------------------------------------ */
+#include <stdint.h>
 
-//C This include is not in upstream neither are the #ifdef __cplusplus
+//C This include is not in upstream
 #include "umm_malloc_cfg.h"   /* user-dependent */
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-void  umm_init( void );
-void *umm_malloc( size_t size );
-void *umm_calloc( size_t num, size_t size );
-void *umm_realloc( void *ptr, size_t size );
-void  umm_free( void *ptr );
+/* ------------------------------------------------------------------------ */
+
+extern void  umm_init( void );
+extern void *umm_malloc( size_t size );
+extern void *umm_calloc( size_t num, size_t size );
+extern void *umm_realloc( void *ptr, size_t size );
+extern void  umm_free( void *ptr );
+
+/* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
 }
 #endif
 
-/* ------------------------------------------------------------------------ */
-
 #endif /* UMM_MALLOC_H */
