@@ -114,33 +114,20 @@ bool MDNSResponder::begin(const char* p_pcHostname, const IPAddress& p_IPAddress
                 IPAddress sta = WiFi.localIP();
                 IPAddress ap = WiFi.softAPIP();
 
-                if (!sta.isSet() && !ap.isSet())
+                if (sta.isSet())
                 {
-
-                    DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] internal interfaces (STA, AP) are not set (none was specified)\n")));
-                    return false;
+                    DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] STA interface selected\n")));
+                    ipAddress = sta;
                 }
-
-                if (ap.isSet())
+                else if (ap.isSet())
                 {
-
-                    if (sta.isSet())
-                    {
-                        DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] default interface AP selected over STA (none was specified)\n")));
-                    }
-                    else
-                    {
-                        DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] default interface AP selected\n")));
-                    }
+                    DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] AP interface selected\n")));
                     ipAddress = ap;
-
                 }
                 else
                 {
-
-                    DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] default interface STA selected (none was specified)\n")));
-                    ipAddress = sta;
-
+                    DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] standard interfaces are not up, please specify one in ::begin()\n")));
+                    return false;
                 }
 
                 // continue to ensure interface is UP
@@ -215,18 +202,26 @@ bool MDNSResponder::begin(const char* p_pcHostname, const IPAddress& p_IPAddress
 */
 bool MDNSResponder::close(void)
 {
+    bool    bResult = false;
 
-    m_GotIPHandler.reset();			// reset WiFi event callbacks.
-    m_DisconnectedHandler.reset();
+    if (0 != m_pUDPContext)
+    {
+        m_GotIPHandler.reset();			// reset WiFi event callbacks.
+        m_DisconnectedHandler.reset();
 
-    _announce(false, true);
-    _resetProbeStatus(false);   // Stop probing
+        _announce(false, true);
+        _resetProbeStatus(false);   // Stop probing
+        _releaseServiceQueries();
+        _releaseUDPContext();
+        _releaseHostname();
 
-    _releaseServiceQueries();
-    _releaseUDPContext();
-    _releaseHostname();
-
-    return true;
+        bResult = true;
+    }
+    else
+    {
+        DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] close: Ignoring call to close!\n")););
+    }
+    return bResult;
 }
 
 /*
@@ -772,6 +767,12 @@ uint32_t MDNSResponder::queryService(const char* p_pcService,
                                      const char* p_pcProtocol,
                                      const uint16_t p_u16Timeout /*= MDNS_QUERYSERVICES_WAIT_TIME*/)
 {
+    if (0 == m_pUDPContext)
+    {
+        // safeguard against misuse
+        return 0;
+    }
+
     DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] queryService '%s.%s'\n"), p_pcService, p_pcProtocol););
 
     uint32_t    u32Result = 0;
